package stx.libjava.tools.source;

import org.eclipse.jdt.core.compiler.InvalidInputException;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.DefaultErrorHandlingPolicies;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class JavaSourceHighlighter extends JavaSourceParser {
    
    protected JavaSourceMarker marker;
    
    public JavaSourceHighlighter() {
        this(new JavaSourceMarker() {            
            @Override
            public void mark(int kind, int from, int to) {
                // TODO Auto-generated method stub
                
            }
        });
    }
    
    public JavaSourceHighlighter(JavaSourceMarker marker) {
        this(
                new ProblemReporter(
                        DefaultErrorHandlingPolicies.proceedWithAllProblems(), 
                        new CompilerOptions(), 
                        new DefaultProblemFactory()),
                true, 
                marker);
    }

    public JavaSourceHighlighter(ProblemReporter problemReporter,
            boolean optimizeStringLiterals, JavaSourceMarker marker) {
        super(problemReporter, optimizeStringLiterals);
        setMarker(marker);
    }
    
    public JavaSourceMarker getMarker() {
        return marker;
    }

    public void setMarker(JavaSourceMarker marker) {
        this.marker = marker;        
        ((Scanner)this.scanner).marker = marker;
    }

    public void initializeScanner(){
        this.scanner = new JavaSourceHighlighter.Scanner(
            false /*comment*/,
            false /*whitespace*/,
            false, /* will be set in initialize(boolean) */
            this.options.sourceLevel /*sourceLevel*/,
            this.options.complianceLevel /*complianceLevel*/,
            this.options.taskTags/*taskTags*/,
            this.options.taskPriorities/*taskPriorities*/,
            this.options.isTaskCaseSensitive/*taskCaseSensitive*/);
    }
        
    public static class Scanner extends org.eclipse.jdt.internal.compiler.parser.Scanner {
        
        public JavaSourceMarker marker;
        protected int lastCommentPtr = -1;

        public Scanner(boolean b, boolean c, boolean d, long sourceLevel,
                long complianceLevel, char[][] taskTags,
                char[][] taskPriorities, boolean isTaskCaseSensitive) {
            super(b,c,d,sourceLevel, complianceLevel, taskTags, taskPriorities, isTaskCaseSensitive);
        }
        
        public void recordComment(int token) {            
            // compute position
            int start = this.startPosition;
            int stop = this.currentPosition;
            super.recordComment(token);
            
            switch (token) {
            case TokenNameCOMMENT_LINE:
            case TokenNameCOMMENT_BLOCK:
                marker.mark(JavaSourceMarker.MARK_COMMENT, start, stop);
                break;                 
            case TokenNameCOMMENT_JAVADOC:
                marker.mark(JavaSourceMarker.MARK_JAVADOC, start, stop);
                break;             
            }
            
        }
        
        
        public int getNextToken() throws InvalidInputException {
            int t = super.getNextToken();
                               
            switch (t) {
            
            case TokenNamebreak:
            case TokenNamecatch:
            case TokenNameelse:    
            case TokenNamefinally:
            case TokenNamecontinue:
            case TokenNamedefault:
            case TokenNamedo:
            case TokenNamefor:
            case TokenNameif:
            case TokenNamecase:
            case TokenNamereturn:
                marker.mark(JavaSourceMarker.MARK_KEYWORD_FLOW, getCurrentTokenStartPosition(), getCurrentTokenEndPosition());
                break;                                            
            case TokenNameassert:
            case TokenNameboolean:            
            case TokenNamebyte:            
            case TokenNamechar:
            case TokenNameclass:
            case TokenNamedouble:            
            case TokenNameextends:
            case TokenNamefalse:
            case TokenNamefinal:
            case TokenNamethrow:
            case TokenNametry:
            case TokenNamewhile:
            case TokenNameswitch:
            case TokenNamefloat:
            case TokenNameimplements:
            case TokenNameimport:
            case TokenNameinstanceof:
            case TokenNameinterface:
            case TokenNamelong:
            case TokenNamenative:
            case TokenNamenew:
            case TokenNamenull:
            case TokenNamepackage:
            case TokenNameprivate:
            case TokenNameprotected:
            case TokenNamepublic:            
            case TokenNameshort:
            case TokenNamestatic:
            case TokenNamestrictfp:
            case TokenNamesuper:            
            case TokenNamesynchronized:
            case TokenNamethis:            
            case TokenNamethrows:
            case TokenNametransient:
            case TokenNametrue:            
            case TokenNamevoid:
            case TokenNamevolatile:            
                marker.mark(JavaSourceMarker.MARK_KEYWORD, getCurrentTokenStartPosition(), getCurrentTokenEndPosition());
                break;
            case TokenNameIntegerLiteral:
            case TokenNameLongLiteral:
            case TokenNameFloatingPointLiteral:
            case TokenNameDoubleLiteral:
                marker.mark(JavaSourceMarker.MARK_NUMBER, getCurrentTokenStartPosition(), getCurrentTokenEndPosition());
                break;
            case TokenNameStringLiteral:
                marker.mark(JavaSourceMarker.MARK_STRING, getCurrentTokenStartPosition(), getCurrentTokenEndPosition());
                break;
            case TokenNameCharacterLiteral:                
                marker.mark(JavaSourceMarker.MARK_CHARACTER, getCurrentTokenStartPosition(), getCurrentTokenEndPosition());
                break; 
            }
            
            return t;
        }
        
    }
    
}
